################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: bam2wiggle.py 2832 2009-11-24 16:11:06Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
bam2wiggle.py - convert bam to wig/bigwig file
==============================================

:Author: Andreas Heger
:Release: $Id: bam2wiggle.py 2832 2009-11-24 16:11:06Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

convert a bam file to a bigwig or bedgraph file.   

The script requires the executables :file:`wigToBigWig` and 
:file:`bedToBigBed` to be in the user's PATH.

If no --shift or --extend option are given, the coverage is computed directly on reads. 
Counting can be performed at a certain resolution.
The counting currently is not aware of spliced reads, i.e., an inserted intron will be 
included in the coverage.

If --shift or --extend are given, the coverage is computed by shifting read
alignment positions upstream for positive strand reads or downstream for negative
strand reads and extend them by a fixed amount. 

For RNASEQ data it might be best to run genomeCoverageBed directly on the 
bam file.

Usage
-----

Type::

   python bam2wiggle.py --output-format=bigwig in.bam out.bigwig

to convert the :term:`bam` file file:`in.bam` to :term:`bigwig` format 
and save the result in :file:`out.bigwig`.

Type::

   python bam2wiggle.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse
import itertools
import tempfile
import shutil
import subprocess

import CGAT.Experiment as E
import pysam
import CGAT.IOTools as IOTools
# for merging pairs
import bam2bed

class SpanWriter(object):
    '''output values within spans.
    
    values are collected according to a span and an average is
    output.
    '''
    def __init__(self,span):
        self.span = span

        self.laststart = 0
        self.lastend = None
        self.val = 0

        self.lastout = None

    def __call__( self, outfile, contig, start, end, val ):
        
        # deal with previous window
        if self.lastend:
            last_window_start = self.lastend - self.lastend % self.span
            last_window_end = last_window_start + self.span
        else:
            last_window_start = start - start % self.span
            last_window_end = start + self.span

        # print start, end, val, last_window_start, last_window_end

        if self.lastend and start > last_window_end:
            # no overlap, output previous span
            assert self.lastout != last_window_start, \
                "start=%i, end=%i, laststart=%i, lastend=%i, last_window_start=%i" % \
                (start, end, self.laststart, self.lastend, last_window_start)
            self.lastout = last_window_start
            v = self.val / float(self.span)
            outfile.write( "%i\t%f\n" % (last_window_start, v ) )
            self.val = 0

            last_window_start = start - start % self.span
            last_window_end = start + self.span

        if end < last_window_end:
            # window too small to output, simply add values
            self.val += val * (end - start)
            self.lastend = max( end, self.lastend )
        else:
            # output first window
            v = self.val + val * (self.span - start % self.span) / float(self.span)
            
            s = last_window_start
            assert self.lastout != s, \
                    "start=%i, end=%i, laststart=%i, lastend=%i, s=%i" % \
                    (start, end, self.laststart, self.lastend, s)
            outfile.write( "%i\t%f\n" % (s, v ) )
            self.lastout = s
            self.val = 0

            # Output middle windows
            for x in range( start + self.span - start % self.span, end - self.span, self.span ):
                assert self.lastout != x
                outfile.write( "%i\t%f\n" % (x, val ) )
                self.lastout = x

            if end % self.span:
                # save rest
                self.lastend = end 
                self.val = val * end % self.span
            elif end - self.span != last_window_start:
                # special case, end ends on window
                assert self.lastout != end - self.span, \
                    "start=%i, end=%i, laststart=%i, lastend=%i" % \
                    (start, end, self.laststart, self.lastend)
                self.lastout = end - self.span
                outfile.write( "%i\t%f\n" % (end - self.span, val ) )
                self.lastend = None
            else:
                # special case, end ends on window and only single window - already
                # output as start
                self.lastend = None


    def flush( self, outfile ):
        if self.lastend:
            outfile.write( "%i\t%f\n" % (self.lastend - self.lastend % self.span, 
                                         self.val / (self.lastend % self.span) ) )

def main( argv = None ):
    """script main.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: bam2wiggle.py 2832 2009-11-24 16:11:06Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=("bedgraph", "wiggle", "bigbed", "bigwig", "bed"),
                      help="output format [default=%default]" )

    parser.add_option("-b", "--output-filename", dest="output_filename", type="string",
                      help="filename for output [default=%default]" )

    parser.add_option( "-s", "--shift", dest="shift", type = "int",
                       help = "shift reads by a certain amount (ChIP-Seq) [%default]" )

    parser.add_option( "-e", "--extend", dest="extend", type = "int",
                       help = "extend reads by a certain amount (ChIP-Seq) [%default]" )

    parser.add_option( "-p", "--span", dest="span", type = "int",
                       help = "span of a window in wiggle tracks [%default]" )

    parser.add_option("-m", "--merge-pairs", dest="merge_pairs", action="store_true",
                      help="merge paired-ended reads into a single bed interval [default=%default]. " )

    parser.add_option( "--max-insert-size", dest="max_insert_size", type = "int",
                      help = "only merge if insert size less that # bases. 0 turns of this filter [default=%default]."  )

    parser.add_option( "--min-insert-size", dest="min_insert_size", type = "int",
                       help = "only merge paired-end reads if they are at least # bases apart. "
                              " 0 turns of this filter. [default=%default]" )
    parser.set_defaults(
        samfile = None,
        output_format = "wiggle",
        output_filename = None,
        shift = 0,
        extend = 0,
        span = 1,
        merge_pairs = None,
        min_insert_size = 0,
        max_insert_size = 0,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) >= 1:
        options.samfile = args[0]

    if not options.samfile:
        raise ValueError("please provide a bam file")

    if len(args) == 2:
        options.output_filename = args[1]

    samfile = pysam.Samfile( options.samfile, "rb" )

    contig_sizes = dict( zip( samfile.references, samfile.lengths) )

    if options.shift or options.extend:
        if options.output_format != "bigwig":
            raise ValueError( "shift and extend only available for bigwig output" )

    if options.output_format in ("bigwig", "bigbed"):
        
        if not options.output_filename:
            raise ValueError("please output file for bigwig/bigbed computation.")

        if options.output_format == "bigwig":
            executable_name = "wigToBigWig"
        elif options.output_format == "bigbed":
            executable_name = "bedToBigBed"
        else:
            raise ValueError("unknown output format `%s`" % options.output_format)

        executable = IOTools.which( executable_name )

        if not executable:
            raise OSError( "could not find %s in path." % executable_name )

        tmpdir = tempfile.mkdtemp()
        E.debug( "temporary files are in %s" % tmpdir)

        tmpfile_wig = os.path.join( tmpdir, "wig" )
        tmpfile_sizes = os.path.join( tmpdir, "sizes" )

        # write contig sizes
        outfile_size = open( tmpfile_sizes, "w")
        for contig, size in contig_sizes.items():
            outfile_size.write("%s\t%s\n" % (contig, size) )
        outfile_size.close()    
        
        outfile = open( tmpfile_wig, "w" )
        E.info( "starting output to %s" % tmpfile_wig )

    else:
        outfile = options.stdout
        E.info( "starting output to stdout" )        

    if options.output_format in ("wiggle", "bigwig"):
        # wiggle is one-based, so add 1, also step-size is 1, so need to output all bases
        if options.span == 1:
            outf = lambda outfile, contig, start, end, val: \
                outfile.write( "".join( [ "%i\t%i\n" % (x, val) for x in xrange(start+1,end+1) ] ) )
        else:
            outf = SpanWriter( options.span )

    elif options.output_format in ("bed", "bigbed"):
        # bed is 0-based, open-closed
        outf = lambda outfile, contig, start, end, val: \
            outfile.write("%s\t%i\t%i\t%i\n" % (contig, start, end,val))

    output_filename = os.path.abspath( options.output_filename )

    ninput, nskipped, ncontigs = 0, 0, 0

    if options.shift > 0 or options.extend > 0 or options.merge_pairs:

        if options.merge_pairs:
            E.info( "merging pairs to temporary file" )
            counter = bam2bed._bam2bed.merge_pairs( samfile, 
                                                    outfile,
                                                    min_insert_size = options.min_insert_size,
                                                    max_insert_size = options.max_insert_size,
                                                    bed_format = 3)
        else:
            # create bed file with shifted tags
            shift, extend = options.shift, options.extend
            shift_extend = shift + extend

            for contig in samfile.references:
                E.debug("output for %s" % contig )
                lcontig = contig_sizes[contig]

                for read in samfile.fetch( contig ):
                    pos = read.pos
                    if read.is_reverse:
                        start = max(0, read.pos + read.alen - shift_extend )
                    else: 
                        start = max(0, read.pos + shift)

                    # intervals extending beyond contig are removed
                    if start >= lcontig: continue

                    end = min( lcontig, start + extend )
                    outfile.write( "%s\t%i\t%i\n" % (contig, start, end))
                
        outfile.close()

        tmpfile_bed = os.path.join( tmpdir, "bed" )
        E.info("computing coverage")
        # calculate coverage - format is bedgraph
        statement = "genomeCoverageBed -bg -i %(tmpfile_wig)s -g %(tmpfile_sizes)s > %(tmpfile_bed)s" % locals()
        E.run( statement )
        
        E.info("converting to bigwig" )

        tmpfile_sorted = os.path.join( tmpdir, "sorted" )
        statement = "sort -k 1,1 -k2,2n %(tmpfile_bed)s > %(tmpfile_sorted)s; bedGraphToBigWig %(tmpfile_sorted)s %(tmpfile_sizes)s %(output_filename)s" % locals()
        E.run( statement)

        shutil.rmtree( tmpdir )

    else:

        def column_iter( iterator ):
            
            start = None
            end = 0
            n = None
            for t in iterator:
                if t.pos - end > 1 or n != t.n: 
                    if start != None: yield start, end, n
                    start = t.pos
                    end = t.pos
                    n = t.n
                end = t.pos
            yield start, end, n

        for contig in samfile.references:
            # if contig != "chrX": continue
            E.debug("output for %s" % contig )
            lcontig = contig_sizes[contig]

            if options.output_format in ("wiggle", "bigwig"):
                outfile.write( "variableStep chrom=%s span=%i\n" % (contig, options.span) )
                
            for start, end, val in column_iter( samfile.pileup(contig)):

                # patch: there was a problem with bam files and reads overextending at the end.
                # These are usually Ns, but need to check as otherwise wigToBigWig fails.
                if lcontig <= end: 
                    E.warn( "read extending beyond contig: %s: %i > %i" % (contig, end, lcontig))
                    end = lcontig 
                    if start >= end: continue

                if val > 0: 
                    outf( outfile, contig, start, end, val)
            ncontigs += 1

        if type(outf) == type(SpanWriter):
            outf.flush(outfile)

        E.info( "finished output" )

        E.info( "ninput=%i, ncontigs=%i, nskipped=%i" % (ninput, ncontigs, nskipped) )

        if options.output_format in ("bigwig", "bigbed"):
            outfile.close()

            E.info( "starting %s conversion" % executable )
            try:
                retcode = subprocess.call( " ".join( (executable,
                                                      tmpfile_wig,
                                                      tmpfile_sizes,
                                                      output_filename )),
                                           shell=True)
                if retcode != 0:
                    E.warn( "%s terminated with signal: %i" % (executable, -retcode))
                    return -retcode
            except OSError, msg:
                E.warn( "Error while executing bigwig: %s" % e)
                return 1

            shutil.rmtree( tmpdir )

            E.info( "finished bigwig conversion" )

    E.Stop()


if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gtf2Overlap.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Take two gtf files, merge transcripts, intersect them and draw a venn diagram of the overlap.
This is a crude way if assessing the overlap between two gtf files. Results in a locus
intersection. The intersection keeps all records in gtf-a that intersect gtf-b

Usage
-----

Example::

   python gtf2Overlap.py --help

Type::

   python gtf2Overlap.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import itertools
import random
import CGAT.Pipeline as P
import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
from rpy2.robjects import r as R
from rpy2.robjects.vectors import FloatVector
from rpy2.robjects.vectors import StrVector

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-a", "--gtf-a", dest="gtf_a", type="string",
                      help="supply a gtf file - will compress uncompressed files"  )
    parser.add_option("-b", "--gtf-b", dest = "gtf_b", type = "string",
                      help="supply a second gtf file - will compress uncompressed files")
    parser.add_option("-s", "--scripts-dir", dest = "scripts_dir", type = "string",
                      help="supply a location for accessory scripts")
    parser.add_option( "--no-venn", dest = "no_venn", action="store_true", 
                      help="set if no venn is to be drawn")

    
    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    gtf_files = [options.gtf_a, options.gtf_b]

    merged_files = []
    prefices = []
    E.info("merging gtf files")
    for gtf in gtf_files:
        if gtf.endswith(".gtf.gz"):
            outfile = P.snip(gtf, ".gtf.gz") + ".merged.gtf.gz"
            prefices.append(P.snip(gtf, ".gtf.gz"))
            merged_files.append(outfile)
            statement = '''zcat %s | python %s/gtf2gtf.py --merge-transcripts --log=%s.log | gzip > %s''' % (gtf, options.scripts_dir, outfile, outfile)
            P.run()
        elif gtf.endswith(".gtf"):
            outfile = P.snip(gtf, ".gtf") + ".merged.gtf.gz"
            prefices.append(P.snip(gtf,".gtf"))
            merged_files.append(outfile)
            statement = '''cat %s | python %s/gtf2gtf.py --merge-transcripts --log=%s.log | gzip  > %s''' % (gtf, options.scripts_dir, outfile, outfile)
            P.run()
        else:
            raise ValueError("cannot perform merge on %s: is not a gtf file" % gtf)

    for prefix in prefices:
        if options.gtf_a.find(prefix) != -1:
            gtf_a = prefix + ".merged.gtf.gz"
            prefix_a = prefix
        elif options.gtf_b.find(prefix) != -1:
            gtf_b = prefix + ".merged.gtf.gz"
            prefix_b = prefix

    E.info("intersecting gtf files")
    # intersect the resulting merged files

    scriptsdir = options.scripts_dir
    intersection_out = "_vs_".join([prefix_a, prefix_b]) + ".intersection.gtf.gz" 
    statement = '''intersectBed -a %(gtf_a)s -b %(gtf_b)s -s -wa
                 | python %(scriptsdir)s/gtf2gtf.py --merge-transcripts --log=log | gzip > %(intersection_out)s'''
    P.run()

    if not options.no_venn:
        E.info("producing venn diagram for %s vs %s..." % (options.gtf_a, options.gtf_b))
        # produce the venn diagram
        intersection_file = intersection_out
        gtf_a_merged = gtf_a
        gtf_b_merged = gtf_b

        # create dictionary key
        gtf_pair = (gtf_a_merged, gtf_b_merged)

        # containers for counts
        count_gtf_merged_a = 0
        count_gtf_merged_b = 0
        count_intersection = 0

        # create GTF iterator objects
        gtf_iterator_a = GTF.iterator(IOTools.openFile(gtf_pair[0]))
        gtf_iterator_b = GTF.iterator(IOTools.openFile(gtf_pair[1]))
        gtf_iterator_intersection = GTF.iterator(IOTools.openFile(intersection_file))

        # do the counts for each file
        E.info("counting entries in %s" % gtf_a)
        for entry in gtf_iterator_a:
            count_gtf_merged_a += 1
        print "counts for gtf-a: ",count_gtf_merged_a

        E.info("counting entries in %s" % gtf_b)
        for entry in gtf_iterator_b:
            count_gtf_merged_b += 1
        print "counts for gtf-b: ",count_gtf_merged_b

        E.info("counting entries in %s" % intersection_file)
        for entry in gtf_iterator_intersection:
            count_intersection += 1
        print "counts for intersection: ", count_intersection

        # this is the important bit - basically take an arbitrary list of numbers to represent the list of lincrna in the refnoncoding set
        # then use the intersection count to represent the overlapping section in the lincrna set and add a set of random numbers to this 
        # set to make up the remaining - non-overlapping set

        result = {}
        E.info("assembling count lists")
        result[gtf_pair] = {"gtf-b" : map(str,xrange(count_gtf_merged_b))  , "gtf-a" : map(str,xrange(count_intersection)) + map(str, [random.random() for i in range(count_intersection,count_gtf_merged_a)]  )}

        R_source = os.path.join(os.path.abspath(options.scripts_dir), "venn_diagram.R")
        R.source(R_source)

        prefix_a = prefix_a.replace(".", "_").replace("-", "_")
        prefix_b = prefix_b.replace(".", "_").replace("-", "_")
        
        R('''prefix.a <- "%s"''' % prefix_a)
        R('''prefix.b <- "%s"''' % prefix_b) 
        E.info("drawing venn diagram to %s" % (prefix_a + "_vs_" + prefix_b + ".overlap.png"))
        
        R["venn.diagram2"](R.list( A = result[gtf_pair]["gtf-a"], B = result[gtf_pair]["gtf-b"])
        , prefix_a + "_vs_" + prefix_b + ".overlap.png"
        , **{'cat.cex': 1.5
             , 'main.fontfamily': "Arial"
             , 'cat.pos':FloatVector((0,0))
             , 'cat.fontfamily':"Arial"
             , 'main.cex':1.8                                                                                                                                                                                                              
             , 'height':1000
             , 'width':1000
             , 'cex':2                                                                                                                                                                                                                      
             , 'fontfamily':"Arial"                                                                                                                                                                                                         
             , 'lwd':R.c(1,1)                                                                                                                                                                                                               
             , 'fill':R.c(R.rgb(0,0,0.5,0.5), R.rgb(0.5,0,0,0.5))                                                                                                                                                         
             , 'category.names':R.c(prefix_a, prefix_b) 
             , 'margin' : R.c(0.1,0.1,0.1,0.1)
             })

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
go2svg.py - plot results of a GO analysis
=========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script creates a colourful plot in :term:`svg` format from
the results of a GO analysis.

Usage
-----

Example::

   python go2svg.py --help

Type::

   python go2svg.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile


import CGAT.Experiment as E
import CGAT.SVGdraw as SVGdraw
import bisect
import numpy
import CGAT.Stats as Stats

class DataPoint:
    def __init__(self):
        pass

class DataFDR:
    def __init__(self):
        pass

def Collect( infile, 
             with_headers = False, 
             annotator_format = False, 
             use_annotator_fdr = False, 
             delims = "",
             ignore = "",
             max_pvalue = 1.0,
             max_qvalue = None):
    """read input table."""

    data = []

    lines = filter(lambda x: x[0] != "#", infile.readlines())

    if len(lines) == 0: return data
    
    if with_headers:
        del lines[0]

    if annotator_format:        

        lines = [ line for line in lines if not line.startswith( "Iteration" ) ]
        annotator_fdr = {}
        annotator_level = None
        for line in lines:
            if len(line) == 1: continue                  # skip trailing blank lines
            
            if line.startswith( "--" ):
                if line.startswith("-- False"):
                    annotator_level = float(re.search( "-- False Discovery summary for p-value (.+):", line ).groups()[0])
                    annotator_fdr[annotator_level] = {}
                elif line.startswith("--  Category"):
                    pass
                else:
                    if re.search("insufficiently", line): continue
                    dd = re.split("\s+", line[4:-1])
                    d = DataFDR()
                    d.mObserved, d.mAverage, d.mMedian, d.m95 = map(float,dd[1:])
                    annotator_fdr[annotator_level][dd[0]] = d
                continue
            else:
                if line[0] == "Z": continue # skip header
                if len(line[:-1].split('\t')) != 9: continue # HACK: accounts for a bug in Annotator output

                try:
                    (z, percentchange, pvalue, observed, expected, low95, up95, stddev, description) = line[:-1].split('\t')[:9]
                except ValueError:
                    raise ValueError("# parsing error in line: %s" % line[:-1])

            d = DataPoint()
            d.mAnnotation = description
            d.mPValue     = float(pvalue)
            d.mFoldChange = 1.0 + float(percentchange) / 100.0
            data.append(d)
    else:
        
        for line in lines:
            try:
                (code, goid, scount, stotal, spercent, bcount, btotal, bpercent, ratio, pover, punder, goid, category, description) = line[:-1].split("\t")[:14]
            except ValueError:
                raise ValueError("# parsing error in line: %s" % line[:-1])
            
            if code == "+":
                p = pover
            else:
                p = punder

            d = DataPoint()
            d.mAnnotation = description        
            d.mPValue     = float(p)
            d.mFoldChange = float(spercent) / float(bpercent)
            data.append(d)

    # apply filters
    for c in delims:
        for d in data:
            d.mAnnotation = d.mAnnotation.split(c)[0]
    for c in ignore:
        for d in data:
            d.mAnnotation = d.mAnnotation.replace(c, '')

    ninput = len(data)
    no_fdr = False
    ## apply filters

    if ninput > 0:
        if max_qvalue != None:
            if use_annotator_fdr:
                pvalues = annotator_fdr.keys()
                pvalues.sort()
                pvalues.reverse()
                for pvalue in pvalues:
                    try:
                        d = annotator_fdr[pvalue]["Significant"]
                    except KeyError:
                        continue
                    if d.mObserved == 0:
                        E.info("no data remaining after fdr filtering" )
                        data = []
                        break
                    elif d.mAverage / d.mObserved < max_qvalue:
                        E.info("filtering with P-value of %f" % pvalue )
                        data = [ x for x in data if x.mPValue < pvalue ]
                        break
                else:
                    E.warn( "fdr could not be computed - compute more samples (at P = %f, actual fdr=%f)" % (pvalue, d.mAverage / d.mObserved) )
                    no_fdr = True

            if no_fdr:
                if use_annotator_fdr: E.info( "estimating FDR from observed P-Values" )

                pvalues = [x.mPValue for x in data]
                vlambda = numpy.arange(0, max( pvalues), 0.05 )
                try:
                    qvalues = Stats.doFDR( pvalues, vlambda = vlambda, fdr_level = max_qvalue )
                except ValueError, msg:
                    E.warn( "fdr could not be computed - no filtering: %s" % msg)
                    no_fdr = True
                else:
                    data = [ x[0] for x in zip( data, qvalues.mPassed ) if x[1] ]
        elif max_pvalue != None:
            data = [ x for x in data if x.mPValue < max_pvalue ]

    if no_fdr: data = []

    nremoved = ninput - len(data)

    return data, nremoved, no_fdr

# some definitions for the layout of the picture
BLACK = (0,0,0)
WHITE = (255,255,255)
RED   = (255,0,0)
GREEN = (0,255,0)
BLUE  = (0,0,255)
YELLOW = (255,255,0)
CYAN   = (0,255,255)
PURPLE = (255,0,255)
GREY = (128,128,128)

MAX_GREY = 240
GREY_COLORS = map( lambda x: (x,x,x), range(0,MAX_GREY))

DEFAULT_XWIDTH = 500
DEFAULT_YWIDTH = 500
DEFAULT_LINE_DISTANCE = 10

DEFAULT_OFFSET_Y = 20
DEFAULT_BOX_WIDTH = 15
DEFAULT_LINE_SPACING = 40
DEFAULT_TEXT_DISTANCE = 40
DEFAULT_ANNOTATION_DISTANCE = 20
DEFAULT_LINE_WIDTH = 2

DEFAULT_SCALE_FACTOR = 1.0

DEFAULT_COLOR_SEQUENCE   = BLACK
DEFAULT_COLOR_ANNOTATION = BLACK

class GoPlot:

    def __init__(self,
                 row_names,
                 col_names,
                 thresholds_size,
                 thresholds_colour,
                 alt_colours,
                 template = "screen",
                 max_pvalue = None,
                 max_qvalue = None,
                 mark_columns = None,
                 ):
        """If max_pvalue and max_qvalue are given, they are
        added to the footer.

        If mark_columns is given, the columns for which
        mark_columns is True are marked.

        """

        self.mElements = []
        self.mTemplate = template
        
        ## save row names
        self.mRowNames = row_names
        self.mColNames = col_names

        ## colours
        self.startColour  = RED
        self.mMiddleColour = YELLOW
        self.mStopColour   = GREEN
        self.mAltColours = alt_colours

        ## a space
        self.mSeparator = 10
        
        ## info
        self.mMaxPValue = max_pvalue
        self.mMaxQValue = max_qvalue

        ## width and height of a row/column
        self.mRowHeight = 150                               # GAL changed; was 200
        self.mColWidth  = 200

        ## maximum size of a box
        self.mMaxBoxSize = 140
        self.mRevertSize = True

        ## Height and width get set
        self.mHeaderFontSize  = self.mRowHeight / 1.5       # GAL changed; was 2.0
        self.mHeaderFont      = "Verdana"
        
        self.mThresholdsSize = thresholds_size
        self.mThresholdsSizeTitle = "P-Values"
        self.mThresholdsColour = thresholds_colour
        self.mThresholdsColourTitle = "Fold change"

        ## write a grid line every five rows
        self.mRowTicks = 5
        self.mColTicks = 5

        ## footer
        self.mFooterFrom = 10        
        self.mFooterFontSize  = self.mRowHeight / 1.5
        self.mFooterFont      = "Verdana"
        self.mFooter = None

        ## page margin
        self.mBottomMargin = 300
        
        ## Title
        self.mTitleFontSize  = self.mRowHeight
        self.mTitleFont      = "Verdana"
        self.mTitle = None

        if self.mTemplate == "screen":
            ## screen is default
            pass
        elif self.mTemplate == "publication":
            self.startColour  = RED
            self.mMiddleColour = WHITE
            self.mStopColour   = BLUE

        self.mMarkColumns = mark_columns
        if self.mMarkColumns:
            assert len(self.mMarkColumns) == len(self.mColNames), "length of mark_columns must equal length of columns"
            
        self.initializePlot()
        
    #####################################################################                
    def initializePlot( self ):
        """set various coordinates in the plot."""

        ## height of col header
        self.mHeaderHeight = max( map( len, self.mColNames) ) * self.mHeaderFontSize / 2

        if self.mTitle:
            self.mHeaderHeight += self.mSeparator + self.mTitleFontSize
            
        ## width of row header
        self.mHeaderWidth = max( map( len, self.mRowNames) ) * self.mHeaderFontSize / 2
        
        ## height of footer:
        self.mFooterHeight =  2 * max( self.mFooterFontSize, self.mMaxBoxSize) + self.mSeparator

        if self.mFooter:
            self.mFooterHeight += self.mSeparator + self.mFooterFontSize
        
        ## height and width of data section
        self.mDataWidth  = len(col_names) * self.mColWidth 
        self.mDataHeight = len(row_names) * self.mRowHeight

        ## build coordinates for rows and columns
        self.buildMapRow2Position()
        self.buildMapCol2Position()

        ## build colour map
        self.buildColourMap()

        self.mPageWidth = self.mHeaderWidth + self.mDataWidth + 1 * self.mSeparator
        self.mPageHeight = self.mHeaderHeight + self.mDataHeight + self.mFooterHeight + 2 * self.mSeparator + self.mBottomMargin

    #####################################################################
    def setTitle( self, title ):
        """set title."""
        self.mTitle = title

    #####################################################################
    def setFooter( self, footer ):
        """set footer."""
        self.mFooter = footer
        
    #####################################################################
    def writeTitle( self ):
        """write title into plot."""
        
        if self.mTitle:
            e = SVGdraw.text( self.mPageWidth / 2,
                              self.mTitleFontSize ,
                              self.mTitle,
                              self.mTitleFontSize,
                              self.mTitleFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle" )

            self.mElements.append(e)

    #####################################################################        
    def buildColourMap( self ):
        """build map of thresholds to colours.

        This is two gradients:
        1: first half: start to middle
        2: second half: middle to end
        """

        self.mColours = []

        if self.mAltColours:

            for colidx in range(len(self.mThresholdsColour)+1):

                col = colidx / (len(self.mThresholdsColour) - 0.0)

                if col == 0.5:
                    x = 0
                    col0 = [0,0,0]
                    col1 = [0,0,0]
                elif col < 0.5:
                    x = min(1,1-2*col)
                    col0 = [26,0,128]
                    col1 = [255,0,0]
                else:
                    x = min(1,2*col-1)
                    col0 = [26,52,13]
                    col1 = [230,255,52]
                self.mColours.append( (col0[0] + x*(col1[0]-col0[0]),
                                       col0[1] + x*(col1[1]-col0[1]),
                                       col0[2] + x*(col1[2]-col0[2]) ) )

        else:

            num_steps = int(math.floor( (len(self.mThresholdsColour) + 1) / 2.0)) 
            
            d = map( lambda x, y: (x - y) / num_steps, self.mMiddleColour, self.startColour)
            for x in range(num_steps):
                self.mColours.append ( (self.startColour[0] + x * d[0],
                                        self.startColour[1] + x * d[1],
                                        self.startColour[2] + x * d[2] ) )

            # self.mColours.append( self.mMiddleColour )
            
            d = map( lambda x, y: (x - y) / num_steps, self.mStopColour, self.mMiddleColour)
            for x in range(1, num_steps):
                self.mColours.append ( (self.mMiddleColour[0] + x * d[0],
                                        self.mMiddleColour[1] + x * d[1],
                                        self.mMiddleColour[2] + x * d[2] ) )

            self.mColours.append( self.mStopColour )

    #####################################################################        
    def buildMapRow2Position( self ):
        
        ## build map of row_name to row
        self.mMapRow2Position = {}

        offset = self.mHeaderHeight + self.mSeparator
        for x in range(len(self.mRowNames)):
            self.mMapRow2Position[self.mRowNames[x]] = offset + x * self.mRowHeight

    #####################################################################        
    def buildMapCol2Position( self ):

        ## build map of row_name to row
        self.mMapCol2Position = {}

        for x in range(len(self.mColNames)):
            self.mMapCol2Position[self.mColNames[x]] = x * self.mColWidth

    #####################################################################
    def addValue( self, row, col, size, colour_value ):
        """add a dot in row/col.
        """

        ## decide the size of the box
        pos = bisect.bisect( self.mThresholdsSize, size )

        if self.mRevertSize: 
            size = self.mMaxBoxSize * (1.0 - float(pos) / len(self.mThresholdsSize))
        else:
            size = self.mMaxBoxSize * float(pos) / len(self.mThresholdsSize)

        d = (self.mMaxBoxSize - size) / 2
        
        x = self.mMapCol2Position[col] + d
        try: 
            y = self.mMapRow2Position[row] + d
        except KeyError:
            return

        # determine the colour of the box
        pos = bisect.bisect( self.mThresholdsColour, colour_value )
        colour = self.mColours[pos]

        e = SVGdraw.rect( x, y,
                          size, size, 
                          stroke = "black",
                          fill = "rgb(%i,%i,%i)" % colour )

        self.mElements.append(e)
        
    #####################################################################        
    def writeColHeaders( self ):
        """write row headers."""

        current_x = self.mColWidth / 2
        current_y = self.mHeaderHeight
        for i in range( len(self.mColNames) ):
            if self.mMarkColumns and self.mMarkColumns[i]:
                color = BLUE
                name = self.mColNames[i] + "*"
            else:
                color = BLACK
                name = self.mColNames[i]
            
            e = SVGdraw.text( current_x,
                              current_y,
                              name,
                              self.mHeaderFontSize,
                              self.mHeaderFont,
                              stroke = "rgb(%i,%i,%i)" % color,
                              text_anchor = "start",
                              transform="rotate(-45,%i,%i)" %( current_x, current_y ))

            self.mElements.append(e)
            
            current_x += self.mColWidth
            # current_y -= self.mColWidth / 2        # GAL added # AH removed?

    #####################################################################
    def writeGrid( self ):
        """add grid lines."""

        if self.mRowTicks:

            start_x = 0,
            end_x = self.mDataWidth + self.mSeparator + self.mHeaderWidth
            current_y = self.mHeaderHeight + self.mSeparator / 2 + self.mRowTicks * self.mRowHeight
            
            for x in range(self.mRowTicks, len(self.mRowNames), self.mRowTicks):

                e = SVGdraw.line( start_x,
                                  current_y,
                                  end_x,
                                  current_y,
                                  stroke = "rgb(%i,%i,%i)" % BLACK )
                
                self.mElements.append(e)
                
                current_y += self.mRowTicks * self.mRowHeight

        if self.mColTicks:            

            start_y = self.mHeaderHeight + self.mSeparator / 2
            end_y = start_y + self.mDataHeight
            
            current_x = self.mColTicks * self.mColWidth - self.mColWidth / 2
            
            for x in range(self.mColTicks, len(self.mColNames), self.mColTicks):

                e = SVGdraw.line( current_x,
                                  start_y,
                                  current_x,
                                  end_y,
                                  stroke = "rgb(%i,%i,%i)" % BLACK )
                
                self.mElements.append(e)
                
                current_x += self.mColTicks * self.mColWidth
            
    #####################################################################
    def writeRowHeaders( self ):
        """write row headers."""

        current_x = self.mDataWidth + self.mSeparator
        
        current_y = self.mHeaderHeight + self.mSeparator + self.mHeaderFontSize

        for name in self.mRowNames:
            e = SVGdraw.text( current_x,
                              current_y,
                              name,
                              self.mHeaderFontSize,
                              self.mHeaderFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "start")
            
            self.mElements.append(e)
            
            current_y += self.mRowHeight

        self.mHeaderWidth = max( map( len, self.mRowNames) ) * self.mHeaderFontSize / 2
            
    #####################################################################
    def writeFooter( self ):
        """write footer.

        The footer contains the legend.
        """
        current_x = self.mFooterFrom
        current_y = self.mHeaderHeight + self.mDataHeight + 2 * self.mSeparator

        ###########################################################
        ## Draw legend 1: size of boxes
        e = SVGdraw.text( current_x,
                          current_y + self.mFooterFontSize,
                          self.mThresholdsSizeTitle,
                          self.mFooterFontSize,
                          self.mFooterFont,
                          stroke = "rgb(%i,%i,%i)" % BLACK,
                          text_anchor = "start")

        current_x += len(self.mThresholdsSizeTitle) * self.mFooterFontSize / 1.5 + self.mSeparator
        
        self.mElements.append(e)
        
        l = len(self.mThresholdsSize)
        for x in range(l):
            if self.mRevertSize:
                p = int(self.mMaxBoxSize * (1.0 - float(x) / l))
            else:
                p = int(self.mMaxBoxSize * (float(x) / l))
                
            e = SVGdraw.rect( current_x,
                              current_y + (self.mMaxBoxSize - p) / 2,
                              p, p,
                              stroke = "black",
                              fill = "rgb(%i,%i,%i)" % self.startColour )

            self.mElements.append(e)            
            current_x += self.mMaxBoxSize + self.mSeparator 

            t = "< %g" % (self.mThresholdsSize[x])
            e = SVGdraw.text( current_x,
                              current_y + self.mFooterFontSize,
                              t,
                              self.mFooterFontSize,
                              self.mFooterFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "start")                              

            current_x += len(t) * self.mFooterFontSize / 1.5 + self.mSeparator 
            self.mElements.append(e)

        ###########################################################
        ## Draw legend 2: colour of boxes
        current_x = self.mFooterFrom            
        current_y += max( self.mFooterFontSize, self.mMaxBoxSize) + self.mSeparator

        e = SVGdraw.text( current_x,
                          current_y + self.mFooterFontSize,
                          self.mThresholdsColourTitle,
                          self.mFooterFontSize,
                          self.mFooterFont,
                          stroke = "rgb(%i,%i,%i)" % BLACK,
                          text_anchor = "start")

        current_x += len(self.mThresholdsColourTitle) * self.mFooterFontSize / 1.5 + self.mSeparator
        
        self.mElements.append(e)
        
        l = len(self.mThresholdsColour)

        for x in range(l+1):
            
            p = self.mMaxBoxSize

            if x < l:
                t = "< %g" % (self.mThresholdsColour[x])
            else:
                t = "> %g" % (self.mThresholdsColour[x-1])
                
            e = SVGdraw.rect( current_x,
                              current_y,
                              p, p,
                              stroke = "black",
                              fill = "rgb(%i,%i,%i)" % self.mColours[x] )

            self.mElements.append(e)            
            current_x += self.mMaxBoxSize + self.mSeparator 

            e = SVGdraw.text( current_x,
                              current_y + self.mFooterFontSize,
                              t,
                              self.mFooterFontSize,
                              self.mFooterFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "start")                              


            current_x += len(t) * self.mFooterFontSize / 1.5 + self.mSeparator
            self.mElements.append(e)

        ###########################################################
        if self.mMaxPValue != None or self.mMaxQValue != None:
            current_y += max( self.mFooterFontSize / 1.5 , self.mMaxBoxSize) + self.mSeparator
            
            a = []
            if self.mMaxPValue: a.append( "P < %6.4f" % self.mMaxPValue )
            if self.mMaxQValue: a.append( "FDR = %6.4f" % self.mMaxQValue )

            e = SVGdraw.text( self.mPageWidth / 2,
                              current_y + self.mFooterFontSize,
                              " ".join( a ),
                              self.mFooterFontSize,
                              self.mFooterFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle")

        ###########################################################
        if self.mFooter:

            current_y += max( self.mFooterFontSize / 1.5 , self.mMaxBoxSize) + self.mSeparator
            
            e = SVGdraw.text( self.mPageWidth / 2,
                              current_y + self.mFooterFontSize,
                              self.mFooter,
                              self.mFooterFontSize,
                              self.mFooterFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle")
            
            self.mElements.append(e)
            
    #####################################################################        
    def finalizePlot( self ):
        """write remaining parts of the plot."""
        
        self.writeGrid()
        self.writeTitle()
        self.writeFooter()
        self.writeRowHeaders()
        self.writeColHeaders()        

    #####################################################################        
    def writeToFile(self, outfile):
        """write svg image to file.
        """
        self.finalizePlot()

        self.mRoot = SVGdraw.drawing()
        self.mDraw = SVGdraw.svg( (0, 0, self.mPageWidth, self.mPageHeight ) , "100%", "100%" )
        
        for e in self.mElements:
            self.mDraw.addElement( e )
            
        self.mRoot.setSVG(self.mDraw)

        tfile = tempfile.mktemp()
        
        self.mRoot.toXml( tfile )

        lines = open(tfile,"r").readlines()
        
        outfile.write(string.join(lines,""))
        outfile.write("\n")
        
        os.remove(tfile)

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: go2svg.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-e", "--headers", dest="headers", action="store_true",
                      help="first row is a header [ignored]."  )
    parser.add_option("-t", "--title", dest="title", type="string",
                      help="page title.")
    parser.add_option("-f", "--footer", dest="footer", type="string",
                      help="page footer.")
    parser.add_option( "--maxP", dest="max_pvalue", type="float",
                      help="maximum P-value displayed [default=%default].")
    parser.add_option( "--maxQ", dest="max_qvalue", type="float",
                      help="maximum Q-value for controlling for FDR [default=%default].")
    parser.add_option("-c", "--column-titles", dest="col_names", type="string",
                      help="comma separated list of column titles [default: use filenames]."  )
    parser.add_option("-p", "--pattern-filename", dest="pattern_filename", type="string",
                      help="pattern to map columns to filename."  )
    parser.add_option("-A", "--Annotator", dest="annotator", action="store_true",
                      help="use Annotator-style input files.")
    parser.add_option( "--annotator-fdr", dest="annotator_fdr", action="store_true",
                      help="use fdr computed from annotator [default=%default].")
    parser.add_option("-T", "--thresholds", dest="thresholds", type="string",
                      help="7 comma-separated fold-change threshold values")
    parser.add_option("-P", "--pvalues", dest="pvalues", type="string",
                      help="6 comma-separated p value threshold values"),
    parser.add_option("-C", "--altcolours", dest="altcolours", action="store_true",
                      help="Use alternative colour palette")
    parser.add_option("-X", "--delimiters", dest="delims", type="string",
                      help="Delimiter characters for annotation label")
    parser.add_option("-Z", "--ignore", dest="ignore", type="string",
                      help="Ignored characters in annotation label")
    parser.add_option( "--fdr", dest="fdr", type="float",
                      help="filter output by FDR (requires annotator output). [default=%default]")
    parser.add_option("-a", "--template", dest="template", type="choice",
                      choices=("screen", "publication" ),
                      help="layout template to choose - affects colours.")
    parser.add_option( "--sort-columns", dest="sort_columns", type="choice",
                       choices=( "unsorted", "similarity", "alphabetical", ),
                       help="sort columns. The default, unsorted, list columns in the order that they are supplied on the command line [default=%default]")

    parser.set_defaults(
        sortAlphabetically = True,
        headers = False,
        col_names = "",
        pattern_filename = None,
        title = "",
        footer = "",
        max_pvalue = None,
        max_qvalue = None,
        annotator = False,
        thresholds = "0.25,0.33,0.5,1.0,2.0,3.0,4.0",
        pvalues = "0.00001,0.0001,0.001,0.01,0.1",
        altcolours = False,
        delims = "",
        ignore = "",
        template = "screen",
        annotator_fdr = False,
        fdr=None,
        sort_columns = "unsorted",
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if len(args) == 0:
        raise IOError("Please supply at least one input file.")

    if options.pattern_filename:
        input = []
        col_names = args
        for x in col_names: input.append( options.pattern_filename % x )
    else:
        input = args
        
        if options.col_names:
            col_names=options.col_names.split(",")
            if len(col_names) != len(input):
                raise ValueError("Number of col_names and files different: %i != %i" % (len(col_names), len(input)))
        else:
            col_names=input

    E.info( "reading data for %i columns" % len(input))

    columns = []
    errors = []
    
    for col_name, filename in zip( col_names, input ):

        E.debug( "reading data for column %s from %s " % (col_name, filename ))
        # collect all columns
        try:
            values, nremoved, no_fdr = Collect( open(filename,"r"),
                                                with_headers = options.headers,
                                                annotator_format = options.annotator,
                                                delims = options.delims,
                                                ignore = options.ignore,
                                                use_annotator_fdr = options.annotator_fdr,
                                                max_pvalue = options.max_pvalue,
                                                max_qvalue = options.max_qvalue )
        except IOError:
            E.warn( "no data from %s" % filename )
            values = []
            no_fdr = False
            nremoved = 0

        E.info("read %i values from %s: %i significant, %i removed" % (len(values) + nremoved, filename, 
                                                                       len(values),
                                                                       nremoved))
        columns.append( (col_name, values) )
        errors.append( no_fdr )

    if sum( [ len(x) for x in columns]) == 0:
        raise IOError("no data read - please check supplied files.")

    # collect all annotations
    # Also filter for max pvalue
    annotations = set()
    for col_name, column in columns:
        for d in column:
            annotations.add( d.mAnnotation )
                
    E.info( "There are %i rows" % len(annotations) )

    # sort and filter annotations
    # (Code removed which did some filtering; the annotations data is not used)
    # By removing labels from annlist you can select the annotations you want to display
    row_names = list(annotations)
    if options.sortAlphabetically:
        row_names.sort()


    if options.sort_columns == "unsorted":
        pass
    elif options.sort_columns == "alphabetical":
        col_names.sort()
    elif options.sort_columns == "similarity":
        if len(row_names) * len(col_names) > 10000: 
            E.info( "no sorting as matrix too large" )
        else:
            import CorrespondenceAnalysis
            matrix = numpy.ones( (len(row_names), len(col_names)), numpy.float )
            map_rows = dict( zip( row_names, range(len(row_names) ) ) )
            x = 0
            for col_name, column in columns:
                for d in column:            
                    matrix[map_rows[d.mAnnotation],x] = d.mFoldChange
                x += 1
            row_indices, col_indices = CorrespondenceAnalysis.GetIndices( matrix )
            map_row_new2old = numpy.argsort(row_indices)
            map_col_new2old = numpy.argsort(col_indices)
            row_names = [ row_names[map_row_new2old[x]] for x in range(len(row_names))]
            col_names = [ col_names[map_col_new2old[x]] for x in range(len(col_names))]

    E.info( "columns have been sorted" )

    plot = GoPlot( row_names,
                   col_names,
                   thresholds_size = tuple( map( float, options.pvalues.split(',') ) ),
                   thresholds_colour = tuple( map(float, options.thresholds.split(',') ) ),
                   template = options.template, 
                   alt_colours = options.altcolours,
                   max_pvalue = options.max_pvalue, 
                   max_qvalue = options.max_qvalue,
                   mark_columns = errors )

    if options.title: plot.setTitle(options.title)
    if options.footer: plot.setFooter(options.footer)

    plot.initializePlot()
        
    for col_name, column in columns:
        for d in column:
            plot.addValue( d.mAnnotation, 
                           col_name,
                           d.mPValue, 
                           d.mFoldChange )
        
    plot.writeToFile(options.stdout)
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
jalview.py - build annotation for viewing in jalview
==========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script will build annotations for a multiple alignment
such that both can be visualized with :file:`jalview`.

Usage
-----

Example::

   python jalview.py --help

Type::

   python jalview.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import time
import math

import CGAT.Experiment as E
import CGAT.Mali as Mali

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: jalview.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-m", "--method", dest="method", type="choice", 
                      choices=("list2annotation", ),
                      help="methods.")

    parser.add_option("--filename-mali", dest="filename_mali", type="string",
                      help="filename with multiple alignment used for calculating sites - used for filtering" )
    
    parser.add_option("--jalview-title", dest="jalview_title", type="string",
                      help="title for jalview annotation." )
    

    parser.set_defaults(
        method = None,
        jalview_symbol = "*",
        jalview_title = "anno",
        filename_mali = None,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if not options.filename_mali:
        raise "please specify a multiple alignment."
    
    mali = Mali.Mali()
    mali.readFromFile( open(options.filename_mali, "r") )

    if options.method == "list2annotation":

        options.stdout.write("JALVIEW_ANNOTATION\n" )
        options.stdout.write("# Created: %s\n\n" % (time.asctime(time.localtime(time.time()))))

        codes = [""] * mali.getWidth()

        first = True
        for line in sys.stdin:
            if line[0] == "#": continue
            if first:
                first= False
                continue

            position = int(line[:-1].split("\t")[0])
            codes[position-1] = options.jalview_symbol
            
        options.stdout.write("NO_GRAPH\t%s\t%s\n" % (options.jalview_title, "|".join( codes ) ))
            
    E.Stop()



#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fastqs2fastq.py - merge two paired fastq files
===============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes two fastq files and outputs a single fastq file.

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

This script assumes that the fastq files are sorted by read
id.

Code
----

'''

import os
import sys
import re
import optparse
import math
import random

import CGAT.IOTools as IOTools
import CGAT.Experiment as E
import CGAT.Fastq as Fastq

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices = ('reconcile', ),
                      help="method to apply [default=%default]."  )
                      
    parser.add_option("-c", "--chop", dest="chop", action="store_true",
                      help="whether or not to trim last character of sequence name (sometimes used to indicte end) [default=%default]."  )

    parser.add_option("-o", "--output-pattern", dest="output_pattern", type="string",
                      help="pattern for output files [default=%default]."  )

    parser.set_defaults(
        method = "reconcile",
        chop = False,
        output_pattern = "%i.fastq.gz",
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError("please supply at least two fastq files on the commandline")

    fn1, fn2 = args
    c = E.Counter()
    
    if options.method == "reconcile":

        def getIds( infile ):
            '''return ids in infile.'''
            aread = infile.readline
            while True:
                l = [aread().rstrip("\r\n") for i in range(4)]
                if not l[0]: break
                r = l[0].split()[0]
                # decide if to chop read number off
                if options.chop: yield r[:-1]
                else: yield r

        def write( outfile, infile, take ):
            '''filter fastq files with ids in take.'''
            aread = infile.readline
            while True:
                l = [aread().rstrip("\r\n") for i in range(4)]
                if not l[0]: break
                r = l[0].split()[0]
                if options.chop: r = r[:-1]
                if r not in take: continue
                outfile.write("\n".join(l) + "\n" )

        E.info( "reading first in pair" )
        inf1 = IOTools.openFile( fn1 )
        ids1 = set( getIds( inf1 ) )

        E.info( "reading second in pair" )
        inf2 = IOTools.openFile( fn2 )
        ids2 = set( getIds( inf2 ) )

        take = ids1.intersection( ids2 )
        
        E.info( "first pair: %i reads, second pair: %i reads, shared: %i reads" % \
                    ( len(ids1),
                      len(ids2),
                      len(take)) )
 
        with IOTools.openFile( options.output_pattern % 1, "w" ) as outf:
            inf = IOTools.openFile( fn1 )
            E.info( "writing first in pair" )
            write( outf, inf, take )

        with IOTools.openFile( options.output_pattern % 2, "w" ) as outf:
            inf = IOTools.openFile( fn2 )
            E.info( "writing second in pair" )
            write( outf, inf, take )
        
    ## write footer and output benchmark information.
    E.info( "%s" % str(c) )
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree2taxa.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree2taxa.py - extract taxa in a tree
=====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collections of trees from stdin
and for each tree outputs the taxa found within the
tree.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import optparse

from types import *
import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree2taxa.py 2782 2009-09-10 11:40:29Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option( "--skip-trees", dest="skip_trees", action="store_true",
                       help="do not output tree names in third field [default=%default]."  )

    parser.set_defaults(
        skip_trees = False
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    nexus = TreeTools.Newick2Nexus( sys.stdin )
    if options.loglevel >= 1:
        options.stdlog.write( "# read %i trees from stdin.\n" % len(nexus.trees))

    ntree = 0
    ntotal = len(nexus.trees)

    if ntotal == 1:
        options.stdout.write("taxon\n")
    else:
        if options.skip_trees:
            options.stdout.write("taxon\ttree\n")
        else:
            options.stdout.write("taxon\ttree\tname\n")
            
    for tree in nexus.trees:
        ntree += 1
        taxa = TreeTools.GetTaxa( tree )

        if ntotal == 1:
            for t in taxa:
                options.stdout.write("%s\n" % (t))
        elif options.skip_trees:
            for t in taxa:
                options.stdout.write("%s\t%i\n" % (t, ntree))
        else:
            for t in taxa:
                options.stdout.write("%s\t%i\t%s\n" % (t, ntree, tree.name))

    if options.loglevel >= 1:
        options.stdlog.write( "# ntotal=%i\n" % (ntotal))
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
csv_set.py - set operations on a table
======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python csv_set.py --help

Type::

   python csv_set.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

import CGAT.Experiment as E
import CGAT.CSV as CSV
import csv
import hashlib


class UniqueBuffer:
    mKeys = {}
    def __init__(self, outfile):
        self.mOutfile = outfile
    def write( self, out ):
        key = hashlib.md5(out).digest()
        if key not in self.mKeys:
            self.mKeys[key] = True
            self.mOutfile.write(out)
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: csv_set.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-u", "--unique", dest="unique", action="store_true",
                       help="output rows are uniq." )

    parser.add_option( "-1", "--join-fields1", dest="join_fields1", type="string",
                       help="join fields in first table." )
    parser.add_option( "-2", "--join-fields2", dest="join_fields2", type="string",
                       help="join fields in second table." )
    parser.add_option( "-m", "--method", dest="method", type="choice",
                       help="set operation to perform.", choices=("intersection", "rest", "union") )

    parser.set_defaults(
        remove = False,
        unique = False,
        join_fields1 = None,
        join_fields2 = None,
        method = "intersection",
        )

    (options, args) = E.Start( parser, add_csv_options  = True)

    if len(args) != 2:
        raise "please specify two files to join."

    if not options.join_fields1 or not options.join_fields2:
        raise "please specify at least one join field per table."

    options.join_fields1 = options.join_fields1.split(",")
    options.join_fields2 = options.join_fields2.split(",")    
        
    options.filename1, options.filename2 = args

    fields1, table1 = CSV.ReadTable( open(options.filename1, "r") )
    fields2, table2 = CSV.ReadTable( open(options.filename2, "r") )    

    if options.unique:
        outfile = UniqueBuffer(sys.stdout)
    else:
        outfile = options.stdout

    nfields1 = []
    for x in range( len(fields1) ):
        if fields1[x] in options.join_fields1: nfields1.append( x )
    nfields2 = []
    for x in range( len(fields2) ):
        if fields2[x] in options.join_fields2: nfields2.append( x )    
        
    ## calculate row indices: double keys are not taken care of here
    keys = {}
    for row1 in table1:
        v = map( lambda x: row1[x], nfields1)
        key = hashlib.md5("".join(v)).digest()        
        keys[key] = row1

    if options.method == "intersection":
        ## build new field list
        take = range(len(fields1))
        c = len(take)
        for x in fields2:
            if x not in options.join_fields2:
                take.append(c)
            c+=1

        t = fields1 + fields2

        new_fields = map(lambda x: t[x], take)

        print "\t".join(new_fields)
        
        for row2 in table2:
            v = map( lambda x: row2[x], nfields2)
            key = hashlib.md5("".join(v)).digest()        
            if key in keys:
                new_row = keys[key] + row2
                outfile.write( "\t".join(map(lambda x: new_row[x], take )) + "\n" )

    elif options.method == "rest":

        new_fields = fields2
        print "\t".join(new_fields)        

        for row2 in table2:
            v = map( lambda x: row2[x], nfields2)
            key = hashlib.md5("".join(v)).digest()        
            if key not in keys:
                outfile.write( "\t".join(row2) + "\n")
    
    E.Stop()
